from .common import *
from .comm import *
from .memory import *
from .settings_flash import *
from .settings_sram import *
from .status import *
from .i2c import *
from .gpio import *
from .misc import *

if __name__ == '__main__':
    unittest.main(exit=False)



from .common import *

__all__ = ["TestStatusSetParameters"]

class TestStatusSetParameters(MCPTestWithReadMock):
    # command 0x10
    def setUp(self):
        super().setUp()
        self.mcp._read_response.return_value[0x10]
        self.x10 = self.mcp._read_response.return_value

    def test_i2c_cancel_transfer(self):
        for v in I2CCancelTransferResponse:
            self.x10[2] = v
            ret = self.mcp.i2c_cancel_transfer()
            self.assertEqual(self.mcp.dev.write.call_args[0][0][2], 0x10)
            self.assertEqual(ret, v)
    
    def test_i2c_write_speed_ok(self):
        for v in I2CSetSpeedResponse:
            self.x10[3] = v
            for w in range(0,256):
                self.x10[14] = w
                ret = self.mcp.i2c_write_speed(12000000//(w+3))
                self.assertEqual(self.mcp.dev.write.call_args[0][0][3], 0x20)
                self.assertEqual(self.mcp.dev.write.call_args[0][0][4], w)
                self.assertEqual(ret, v)
                self.mcp.i2c_speed = 12000000//(w+3)
                self.assertEqual(self.mcp.dev.write.call_args[0][0][3], 0x20)
                self.assertEqual(self.mcp.dev.write.call_args[0][0][4], w)
    
    def test_i2c_write_speed_fail(self):
        with self.assertRaises(InvalidParameterException):
            self.mcp.i2c_write_speed(46332)
        with self.assertRaises(InvalidParameterException):
            self.mcp.i2c_write_speed(4000001)
    
    def test_i2c_read_speed(self):
        for w in range(0,256):
            self.x10[14] = w
            self.assertEqual(self.mcp.i2c_read_speed(), 12000000//(w+3))
            self.assertEqual(self.mcp.i2c_speed, 12000000//(w+3))

    def test_read_interrupt_flag(self):
        self.do_test_read_func_bool(self.mcp.read_interrupt_flag, 24)
        self.do_test_read_prop_bool("interrupt_flag", 24)
    
    def test_read_adc(self):
        for n in range(3):
            self.do_test_read_func_word(self.mcp.read_adc, 50+2*n, 31, n)
            self.do_test_read_prop_word("adc{:d}_value".format(n), 50+2*n)
    
    def test_read_firmware_version(self):
        self.x10[46:50] = b"ABCD"
        self.assertEqual(self.mcp.read_firmware_version(), "ABCD")
        self.assertEqual(self.mcp.firmware_version, "ABCD")




from .common import *

__all__ = ["TestReadSRAMChipSettings", "TestWriteSRAMChipSettings",
           "TestGPIOFunctionSRAM"]

class TestReadSRAMChipSettings(MCPTestWithReadMock):
    def setUp(self):
        super().setUp()
        self.mcp._read_response.return_value = self.x61
        self.x61[4:] = bytearray(60)
        self.mcp.set_default_memory_target(MemoryType.SRAM)

    def test_read_clock_output_duty_cycle(self):
        self.x61[5] = 0b00011000
        v = self.mcp.read_clock_output_duty_cycle()
        self.assertEqual(v, ClockDutyCycle.Percent_75)
        v = self.mcp.clock_output_duty_cycle
        self.assertEqual(v, ClockDutyCycle.Percent_75)

    def test_read_clock_output_frequency(self):
        self.x61[5] = 0b00000111
        v = self.mcp.read_clock_output_frequency()
        self.assertEqual(v, ClockFrequency.Clock_375kHz)
        v = self.mcp.clock_output_frequency
        self.assertEqual(v, ClockFrequency.Clock_375kHz)

    def test_read_dac_reference_voltage(self):
        self.x61[6] = 0b11000000
        v = self.mcp.read_dac_reference_voltage()
        self.assertEqual(v, ReferenceVoltageValue.Voltage4_096V)
        v = self.mcp.dac_reference_voltage
        self.assertEqual(v, ReferenceVoltageValue.Voltage4_096V)

    def test_read_dac_reference_source(self):
        self.do_test_read_func_bit(self.mcp.read_dac_reference_source, 6, 5)
        self.do_test_read_prop_bit("dac_reference_source", 6, 5)

    def test_read_interrupt_on_falling_edge(self):
        self.do_test_read_func_bit(self.mcp.read_interrupt_on_falling_edge, 7, 6)
        self.do_test_read_prop_bit("interrupt_on_falling_edge", 7, 6)
    
    def test_read_interrupt_on_rising_edge(self):
        self.do_test_read_func_bit(self.mcp.read_interrupt_on_rising_edge, 7, 5)
        self.do_test_read_prop_bit("interrupt_on_rising_edge", 7, 5)

    def test_read_adc_reference_voltage(self):
        self.x61[7] = 0b00011000
        v = self.mcp.read_adc_reference_voltage()
        self.assertEqual(v, ReferenceVoltageValue.Voltage4_096V)
        v = self.mcp.adc_reference_voltage
        self.assertEqual(v, ReferenceVoltageValue.Voltage4_096V)

    def test_read_adc_reference_source(self):
        self.do_test_read_func_bit(self.mcp.read_adc_reference_source, 7, 2)
        self.do_test_read_prop_bit("adc_reference_source", 7, 2)


class TestWriteSRAMChipSettings(MCPTestWithReadMock):
    def setUp(self):
        super().setUp()
        self.x61[4:] = bytearray(60)
        self.mcp._read_response.return_value = self.x61
        self.mcp.set_default_memory_target(MemoryType.SRAM)

    def test_write_clock_output_duty_cycle(self):
        self.mcp.write_clock_output_duty_cycle(ClockDutyCycle.Percent_75)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][2], 0b10011000)
        self.mcp.clock_output_duty_cycle = ClockDutyCycle.Percent_75
        self.assertEqual(self.mcp.dev.write.call_args[0][0][2], 0b10011000)

    def test_write_clock_output_frequency(self):
        self.mcp.write_clock_output_frequency(ClockFrequency.Clock_375kHz)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][2], 0b10000111)
        self.mcp.clock_output_frequency = ClockFrequency.Clock_375kHz
        self.assertEqual(self.mcp.dev.write.call_args[0][0][2], 0b10000111)

    def test_write_dac_reference_voltage(self):
        self.mcp.write_dac_reference_voltage(ReferenceVoltageValue.Voltage4_096V)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][3], 0b10000110)
        self.mcp.dac_reference_voltage = ReferenceVoltageValue.Voltage4_096V
        self.assertEqual(self.mcp.dev.write.call_args[0][0][3], 0b10000110)

    def test_write_dac_reference_source(self):
        self.mcp.write_dac_reference_source(ReferenceVoltageSource.Internal)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][3], 0b10000001)
        self.mcp.dac_reference_source = ReferenceVoltageSource.Internal
        self.assertEqual(self.mcp.dev.write.call_args[0][0][3], 0b10000001)
    
    def test_write_dac(self):
        self.mcp.write_dac(31)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][4], 0b10011111)
        self.mcp.dac_value = 31
        self.assertEqual(self.mcp.dev.write.call_args[0][0][4], 0b10011111)

    def test_write_adc_reference_voltage(self):
        self.mcp.write_adc_reference_voltage(ReferenceVoltageValue.Voltage4_096V)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][5], 0b10000110)
        self.mcp.adc_reference_voltage = ReferenceVoltageValue.Voltage4_096V
        self.assertEqual(self.mcp.dev.write.call_args[0][0][5], 0b10000110)

    def test_write_adc_reference_source(self):
        self.mcp.write_adc_reference_source(ReferenceVoltageSource.Internal)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][5], 0b10000001)
        self.mcp.adc_reference_source = ReferenceVoltageSource.Internal
        self.assertEqual(self.mcp.dev.write.call_args[0][0][5], 0b10000001)

    def test_write_interrupt_on_falling_edge(self):
        self.mcp.write_interrupt_on_falling_edge(True)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][6], 0b10011000)
        self.mcp.interrupt_on_falling_edge = True
        self.assertEqual(self.mcp.dev.write.call_args[0][0][6], 0b10011000)
    
    def test_write_interrupt_on_rising_edge(self):
        self.mcp.write_interrupt_on_rising_edge(True)
        self.assertEqual(self.mcp.dev.write.call_args[0][0][6], 0b10000110)
        self.mcp.interrupt_on_rising_edge = True
        self.assertEqual(self.mcp.dev.write.call_args[0][0][6], 0b10000110)
    
    def test_clear_interrupt_flag(self):
        self.mcp.clear_interrupt_flag()
        self.assertEqual(self.mcp.dev.write.call_args[0][0][6], 0b10000001)


class TestGPIOFunctionSRAM(MCPTestWithReadMock):
    # command 0x50
    def setUp(self):
        super().setUp()
        self.x61[4:] = bytearray(60)
        self.mcp._read_response.return_value = self.x61
        self.mcp.set_default_memory_target(MemoryType.SRAM)
    
    def test_write_gpio_function(self):
        for pin in range(4):
            value = getattr(mcp2221.enums, "GPIO{:d}Function".format(pin))(2)
            getattr(self.mcp, "gpio{:d}_write_function".format(pin))(value)
            pin_args = self.mcp.dev.write.call_args_list[-2][0][0]
            self.assertEqual(pin_args[7], 0b10000000)
            self.assertEqual(pin_args[8+pin], 0b00000010)
            setattr(self.mcp, "gpio{:d}_function".format(pin), value)
            self.assertEqual(pin_args[7], 0b10000000)
            self.assertEqual(pin_args[8+pin], 0b00000010)
    
    def test_read_gpio_function(self):
        for pin in range(4):
            self.x61[22+pin] = 0b00000010
            expected = getattr(mcp2221.enums, "GPIO{:d}Function".format(pin))(2)
            v = getattr(self.mcp, "gpio{:d}_read_function".format(pin))()
            self.assertEqual(v, expected)
            v = getattr(self.mcp, "gpio{:d}_function".format(pin))
            self.assertEqual(v, expected)



"""Enums for MCP2221 driver."""

__all__ = ["ClockDutyCycle", "ReferenceVoltageValue", "GPIO0Function", "GPIO1Function",
           "GPIO2Function", "GPIO3Function", "I2CCancelTransferResponse", "I2CSetSpeedResponse",
           "FlashDataSubcode", "SramDataSubcode", "I2CMode", "GPIODirection",
           "SecurityOption", "MemoryType", "ClockFrequency", "ReferenceVoltageSource"]

import enum

class ClockDutyCycle(enum.IntEnum):
    """Clock output duty cycle. This is the duty cycle
    of the clock signal when output to GPIO 1. Values are in percent.
    """
    Percent_0 = 0x00
    Percent_25 = 0x01
    Percent_50 = 0x02
    Percent_75 = 0x03

class ReferenceVoltageValue(enum.IntEnum):
    """Voltage reference for ADCs or DAC. Values are in V (replace underscore
    with dot, e.g. 1_024V => 1.024V).
    """
    VoltageOff = 0x00
    Voltage1_024V = 0x01
    Voltage2_048V = 0x02
    Voltage4_096V = 0x03

class ClockFrequency(enum.IntEnum):
    """Clock output frequency. This is the frequency
    of the clock signal when output to GPIO 1.
    """
    Clock_24MHz = 0x01
    Clock_12MHz = 0x02
    Clock_6MHz = 0x03
    Clock_3MHz = 0x04
    Clock_1_5MHz = 0x05
    Clock_750kHz = 0x06
    Clock_375kHz = 0x07

class GPIO0Function(enum.IntEnum):
    """Functions for GPIO pin 0. These are:
        GPIO: normal GPIO operation
        USBSuspendState: indicates if chip is in suspend state
        UartRxLed: activity signal for UART RX
    """
    GPIO = 0x00
    USBSuspendState = 0x01
    UartRxLed = 0x02

class GPIO1Function(enum.IntEnum):
    """Functions for GPIO pin 1. These are:
        GPIO: normal GPIO operation
        ClockOutput: clock output
        ADC1: input for ADC1
        UartTxLed: activity signal for UART TX
        Interrupt: input for interrupt trigger
    """
    GPIO = 0x00
    ClockOutput = 0x01
    ADC1 = 0x02
    UartTxLed = 0x03
    Interrupt = 0x04

class GPIO2Function(enum.IntEnum):
    """Functions for GPIO pin 2. These are:
        GPIO: normal GPIO operation
        USBConfigured: indicates if USB connection is configured
        ADC2: input for ADC2
        DAC1: output for DAC
    """
    GPIO = 0x00
    USBConfigured = 0x01
    ADC2 = 0x02
    DAC1 = 0x03

class GPIO3Function(enum.IntEnum):
    """Functions for GPIO pin 3. These are:
        GPIO: normal GPIO operation
        I2CLed: activity signal for I2C
        ADC3: input for ADC3
        DAC2: output for DAC
    """
    GPIO = 0x00
    I2CLed = 0x01
    ADC3 = 0x02
    DAC2 = 0x03

class I2CCancelTransferResponse(enum.IntEnum):
    """Response for I2C cancel transfer command. These are:
        NoOp: not in the right state to do anything
        MarkedForCancellation: transfer is marked for cancellation
        InIdleMode: I2C bus is already in idle state
    """
    NoOp = 0x00
    MarkedForCancellation = 0x10
    InIdleMode = 0x11

class I2CSetSpeedResponse(enum.IntEnum):
    """Response for I2C set speed command. These are:
        NoOp: not in the right state to do anything
        SpeedConsidered: chip acknowledged request
        SpeedNotSet: chip rejected request
    """
    NoOp = 0x00
    SpeedConsidered = 0x20
    SpeedNotSet = 0x21


class FlashDataSubcode(enum.IntEnum):
    """Codes for access to flash memory registers:
        ChipSettings: general chip settings
        GPSettings: GPIO settings
        USBManufacturerDescriptorString: USB manufacturer descriptor
        USBProductDescriptorString: USB product descriptor
        USBSerialNumberDescriptorString: USB serial number descriptor
        ChipFactorySerialNumber: chip serial number
    """
    ChipSettings = 0x00
    GPSettings = 0x01
    USBManufacturerDescriptorString = 0x02
    USBProductDescriptorString = 0x03
    USBSerialNumberDescriptorString = 0x04
    ChipFactorySerialNumber = 0x05

class SramDataSubcode(enum.IntEnum):
    """Codes for access to SRAM registers:
        ChipSettings: general chip settings
        GPSettings: GPIO settings
    """
    ChipSettings = 0x00
    GPSettings = 0x01

class I2CMode(enum.IntEnum):
    """I2C data transfer modes.
    These are defined in the I2C protocol.
    """
    Start = 0x90
    RepeatedStart = 0x92
    NoStop = 0x94

class GPIODirection(enum.IntEnum):
    """GPIO direction values."""
    Output = 0x00
    Input = 0x01
    NotSet = 0xef

class SecurityOption(enum.IntEnum):
    """Chip security options."""
    Unsecured = 0x00
    PasswordProtected = 0x01
    PermanentlyLocked = 0x02

class MemoryType(enum.IntEnum):
    """Memory type. This tells which memory should be accessed
    for values present in both SRAM and flash.
    """
    SRAM = 0x00
    Flash = 0x01

class ReferenceVoltageSource(enum.IntEnum):
    """Voltage reference source for ADCs and DAC."""
    Vdd = 0x00
    Internal = 0x01



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pymcp2221",
    version="1.0.10",
    author="Vincent Paeder",
    author_email="python@paeder.fi",
    description="Python driver for the Microchip MCP2221/MCP2221A USB 2.0 to I2C/UART protocol converters.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=['MCP2221', 'MCP2221A', 'Microchip'],
    url="https://github.com/vpaeder/pymcp2221",
    project_urls={
        "Bug Tracker": "https://github.com/vpaeder/pymcp2221/issues",
    },
    packages=setuptools.find_packages(),
    install_requires=['hidapi'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Communications",
        "Topic :: System :: Hardware :: Hardware Drivers",
        "Topic :: System :: Hardware :: Universal Serial Bus (USB)",
        "Topic :: System :: Hardware :: Universal Serial Bus (USB) :: Human Interface Device (HID)"
    ],
    python_requires='>=3.2',
)
